function [bbs,scores,orgbbind,E] = edgeBoxesScoreWindows(I,model,bbs,doRefinement,varargin)
% Score a predefined set of bounding boxes using the edgeBoxes model. 
% The purpose is NOT to detect boxes, but just get objectness scores.
% Optionally does local greedy search for box refinement.
% Does NOT do NMS.
%
% INPUT
% I
% model
% bbs            (nbox 4), each row is [x y width height] = [column row width height] (in matlab coordinates)
% doRefinement   Boolean, whether to refine boxes or not.
% opts           Additional parameters (struct or name/value pairs), safe to ignore and leave at default vals
%   .alpha          - [.65] step size for local refinement
%   .edgeMinMag     - [.1] increase to trade off accuracy for speed
%   .edgeMergeThr   - [.5] increase to trade off accuracy for speed
%   .clusterMinMag  - [.5] increase to trade off accuracy for speed
%   .gamma          - [2] affinity sensitivity, see equation 1 in paper
%   .kappa          - [1.5] scale sensitivity, see equation 3 in paper
%
% OUTPUT
% bbs
% scores        Edge-box scores
% orgbbind      Corresponding original bounding box indices (in case some boxes are suppressed)
% E 
%
% Gokberk Cinbis, Oct 2014, Based on edgeBoxes.m and other source codes inthe edgeBoxes package.

% get default parameters (unimportant parameters are undocumented)
dfs={'alpha',.65,'beta',.75,'edgeMinMag',.1, 'edgeMergeThr',.5, 'clusterMinMag',.5, ...
  'gamma',2, 'kappa',1.5 };
o=getPrmDflt(varargin,dfs,1);

if(all(ischar(I))), I=imread(I); end

model.opts.nms=0;
[E,O]=edgesDetect(I,model);

if(0), E=gradientMag(convTri(single(I),4)); E=E/max(E(:)); end

E=edgesNmsMex(E,O,2,0,1,model.opts.nThreads);

% -- important variables, that affect the way the mex file works --
% do NOT change these constants!
o.minScore = -1e9; % do not truncate low-scoring boxes to score 0.

% -- unused variables, assigning default no-op variables just in case --
o.maxBoxes = 0; 
o.maxAspectRatio = 1e9;  
o.minBoxArea = 0; 
% beta: nms threshold for object proposals, unused, but just in case..

% alpha->bbs
% beta->doRefinement
org_nbox = size(bbs,1);
[bbs,scores,orgbbind]=edgeBoxesScoreWindowsMex(E,O,o.alpha,o.beta,o.minScore,o.maxBoxes,...
  o.edgeMinMag,o.edgeMergeThr,o.clusterMinMag,...
  o.maxAspectRatio,o.minBoxArea,o.gamma,o.kappa,double(bbs),doRefinement);

assert(all(orgbbind>0));
assert(all(orgbbind<=org_nbox));



