// GOKBERK: My interface..

#include "mex.h"
#include "edgeBoxesCommon.hpp"

#define arg_bbs          13
#define arg_doRefinement 14

// Matlab entry point: bbs = mex( E, O, prm1, prm2, ... )
void mexFunction( int nl, mxArray *pl[], int nr, const mxArray *pr[] )
{
  // check and get inputs
  if(nr != 15) mexErrMsgTxt("Fifteen inputs required.");
  if(nl != 3)  mexErrMsgTxt("Two outputs expected.");
  if(mxGetClassID(pr[0])!=mxSINGLE_CLASS)  mexErrMsgTxt("E must be a float*");
  if(mxGetClassID(pr[1])!=mxSINGLE_CLASS)  mexErrMsgTxt("O must be a float*");
  arrayf E; E._x = (float*) mxGetData(pr[0]);
  arrayf O; O._x = (float*) mxGetData(pr[1]);
  int h = (int) mxGetM(pr[0]); O._h=E._h=h;
  int w = (int) mxGetN(pr[0]); O._w=E._w=w;

  // GOKBERK
  if(mxGetClassID(pr[arg_bbs])!=mxDOUBLE_CLASS)  mexErrMsgTxt("bbs must be in double");
  if(mxGetNumberOfDimensions(pr[arg_bbs])!=2)    mexErrMsgTxt("bbs must be 2D");
  if(mxGetN(pr[arg_bbs])!=4)                     mexErrMsgTxt("bbs must have size (#windows 4)");
  if(mxGetClassID(pr[arg_doRefinement])!=mxLOGICAL_CLASS) mexErrMsgTxt("doRefinement option must be a scalar logical");
  if(mxGetNumberOfElements(pr[arg_doRefinement])!=1)      mexErrMsgTxt("doRefinement option must be a scalar logical");

  // get input boxes 
  const double *bbs = (double *) mxGetData(pr[arg_bbs]);
  const size_t n = mxGetM(pr[arg_bbs]); // number of boxes
  Boxes boxes;
  boxes.reserve(n); // allocate memory
  for(int i=0; i<n; i++) {
      Box tmp;
      tmp.c      = (int) bbs[i + 0*n]-1; 
      tmp.r      = (int) bbs[i + 1*n]-1; 
      tmp.w      = (int) bbs[i + 2*n]; 
      tmp.h      = (int) bbs[i + 3*n]; 
      tmp.orgbbi = i+1;
      tmp.s      = -1e9; // init to a very low value
      boxes.push_back(tmp);
  }
  const bool doRefinement = float(mxGetScalar(pr[arg_doRefinement])); // old: _beta, option to refine boxes

  // setup and run EdgeBoxGenerator
  arrayf V;
  EdgeBoxGenerator edgeBoxGen; 
  edgeBoxGen._alpha = float(mxGetScalar(pr[2]));
  edgeBoxGen._beta = float(mxGetScalar(pr[3]));
  edgeBoxGen._minScore = float(mxGetScalar(pr[4]));
  edgeBoxGen._maxBoxes = int(mxGetScalar(pr[5]));
  edgeBoxGen._edgeMinMag = float(mxGetScalar(pr[6]));
  edgeBoxGen._edgeMergeThr = float(mxGetScalar(pr[7]));
  edgeBoxGen._clusterMinMag = float(mxGetScalar(pr[8]));
  edgeBoxGen._maxAspectRatio = float(mxGetScalar(pr[9]));
  edgeBoxGen._minBoxArea = float(mxGetScalar(pr[10]));
  edgeBoxGen._gamma = float(mxGetScalar(pr[11]));
  edgeBoxGen._kappa = float(mxGetScalar(pr[12]));
  edgeBoxGen.scoreWindows( boxes, E, O, V, doRefinement );

  // create output bbs and output to Matlab
  int n_ = (int) boxes.size();
  if (n_ != n)
      mexErrMsgTxt("something went wrong");
  pl[0] = mxCreateNumericMatrix(n,4,mxDOUBLE_CLASS,mxREAL);
  pl[1] = mxCreateNumericMatrix(n,1,mxDOUBLE_CLASS,mxREAL);
  pl[2] = mxCreateNumericMatrix(n,1,mxDOUBLE_CLASS,mxREAL);
  double *out_bbs = (double*) mxGetData(pl[0]);
  double *out_scores = (double*) mxGetData(pl[1]);
  double *out_bbind = (double*) mxGetData(pl[2]);
  for(int i=0; i<n; i++) {
    out_bbs[ i + 0*n ] = (double) boxes[i].c+1;
    out_bbs[ i + 1*n ] = (double) boxes[i].r+1;
    out_bbs[ i + 2*n ] = (double) boxes[i].w;
    out_bbs[ i + 3*n ] = (double) boxes[i].h;
    out_scores[  i   ] = boxes[i].s;
    out_bbind[  i   ] = boxes[i].orgbbi;
  }
}
